@@ -185,30 +185,25 @@ sub insert_many
         }
     }
 
-    my $wantarray = wantarray;
-
     my @bind_attributes = $class->_bind_attributes_for( $dbh, @non_literal_row_keys );
 
+    my $wantarray = wantarray;
+
     my @objects;
     for my $row (@rows)
     {
-        $class->_sth_execute( $sth,
-                              [ map { $class->_deflated_value( $_, $row->{$_} ) }
-                                @non_literal_row_keys ],
-                              \@bind_attributes,
-                            );
-
-        next unless defined $wantarray;
-
-        for my $col (@auto_inc_columns)
-        {
-            $row->{$col} = $dbh->last_insert_id( undef, undef, $table_name, $col );
-        }
-
-        delete @{ $row }{ @ref_row_keys }
-            if @ref_row_keys;
-
-        push @objects, $class->new( %{ $row }, _from_query => 1 );
+        push @objects,
+            $class->_insert_one_row
+                ( $row,
+                  $dbh,
+                  $sth,
+                  \@non_literal_row_keys,
+                  \@ref_row_keys,
+                  \@bind_attributes,
+                  \@auto_inc_columns,
+                  $table_name,
+                  $wantarray,
+                );
     }
 
     return $wantarray ? @objects : $objects[0];
@@ -232,6 +227,41 @@ sub _bind_attributes_for
     return @attr;
 }
 
+sub _insert_one_row
+{
+    my $class                = shift;
+    # This is really grotesque
+    my $row                  = shift;
+    my $dbh                  = shift;
+    my $sth                  = shift;
+    my $non_literal_row_keys = shift;
+    my $ref_row_keys         = shift;
+    my $bind_attributes      = shift;
+    my $auto_inc_columns     = shift;
+    my $table_name           = shift;
+    my $wantarray            = shift;
+
+    $class->_sth_execute( $sth,
+                          [ map { $class->_deflated_value( $_, $row->{$_} ) }
+                            @{ $non_literal_row_keys } ],
+                          $bind_attributes,
+                        );
+
+    return unless defined $wantarray;
+
+    my %auto_inc;
+    for my $col ( @{ $auto_inc_columns } )
+    {
+        $auto_inc{$col} =
+            $dbh->last_insert_id( undef, undef, $table_name, $col );
+    }
+
+    delete @{ $row }{ @{ $ref_row_keys } }
+        if @{ $ref_row_keys };
+
+    return $class->new( %{ $row }, %auto_inc, _from_query => 1 );
+}
+
 sub _sth_execute
 {
     my $self = shift;
@@ -366,7 +396,7 @@ sub _get_column_value
     my $self = shift;
 
     my $col_values = $self->_get_column_values( $self->meta()->_select_by_pk_sql(),
-                                                [ $self->_pk_vals() ],
+                                                [ $self->pk_values_list() ],
                                               );
 
     my $name = shift;
@@ -431,22 +461,31 @@ sub _dbh
     return $source->dbh();
 }
 
-sub _pk_vals
+sub pk_values_hash
 {
     my $self = shift;
 
-    my @pk;
+    my @vals = $self->pk_values_list()
+        or return;
 
-    for my $col_name ( map { $_->name() } @{ $self->Table()->primary_key() } )
-    {
-        my $pred = 'has_' . $col_name;
+    my @cols =
+        ( map { $_->name() }
+          @{ $self->Table()->primary_key() }
+        );
 
-        return unless $self->$pred();
+    return map { $_ => $self->_deflated_value($_) } @cols;
+}
 
-        push @pk, $self->$col_name();
-    }
+sub pk_values_list
+{
+    my $self = shift;
 
-    return @pk;
+    my @cols =
+        ( map { $_->name() }
+          @{ $self->Table()->primary_key() }
+        );
+
+    return map { $self->_deflated_value($_) } @cols;
 }
 
 sub _MakeSelectByPKSQL
@@ -649,6 +688,27 @@ The object is still usable after this method is called, but if you
 attempt to call any method that tries to access the DBMS it will
 probably blow up.
 
+=head2 $object->pk_values_hash()
+
+Returns a hash representing the names and values for the object's
+primary key. The values are returned in their raw form, regardless of
+any transforms specific for a primary key column.
+
+This may return an empty hash if the primary key for the object has
+not yet been determined. This can happen if you try to call this
+method on an object before its attributes have been fetched from the
+dbms.
+
+=head2 $object->pk_values_list()
+
+Returns a list of values for the object's primary key. The values are
+returned in the same order as C<< $self->primary_key() >> returns the
+columns. The values are returned in their raw form, regardless of any
+transforms specific for a primary key column.
+
+This may return an empty list if the primary key for the object
+has not yet been determined.
+
 =head1 METHODS FOR SUBCLASSES
 
 Since your table-based class will be a subclass of this object, there
@@ -665,13 +725,6 @@ $source->dbh() >> on the source.
 
 If there is no source for the given SQL, it will die.
 
-=head2 $object->_pk_vals()
-
-This method returns an array of primary key values for the object's
-row. This may return an empty array if the primary key for the object
-has not yet been determined. This can happen if you try to call this
-on an object before its attributes have been fetched from the dbms.
-
 =head2 $object->_load_from_dbms($params)
 
 This method will be called as part of object construction (unless
@@ -682,7 +735,6 @@ by looking at each of the table's candidate keys in turn. If the
 parameters passed to the constructor include values for all parts of a
 key, it does a select to find a matching row.
 
-
 You can override this method in order to attempt to load an object
 based on some other method. For example, if your user table stores a
 username and a hashed password, you could accept an I<unhashed>
